#!/bin/bash
#
# gitlab-tools:bin/gitlab-bash-api/init.sh
#
if [ "$( type -t is_variable_define_and_not_empty_safe )" != 'function' ] ; then
  echo 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function is_variable_define_and_not_empty_safe not found.' >&2
  exit 1
fi
if [ "$( type -t exit_if_command_is_missing )" != 'function' ] ; then
  echo 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function exit_if_command_is_missing not found.' >&2
  exit 1
fi

function load_configuration {
  # python2 "${PROJECT_HOME}/bin/py2/ini2json.py" "${PYTHON_GITLAB_CONFIG_FILE}" | jq -c .
  python3 "${PROJECT_HOME}/bin/py3/ini2json.py" "${PYTHON_GITLAB_CONFIG_FILE}" | jq -c .
}

function get_python_gitlab_value {
  local -r name="$1"
  local __value__=

  if ! is_variable_define_and_not_empty_safe 'gitlab' ; then
    __value__="$( jq -r ".global.${name} // empty" <<<"${PYTHON_GITLAB_CONFIG}" )" || exit 1

    if [ -z "${__value__}" ] ; then
      local __section__=
      __section__="$( jq -r '.global.default'  <<<"${PYTHON_GITLAB_CONFIG}" )" || exit 1

      __value__="$(
        jq -r ".[\"${__section__}\"].${name} // empty" <<<"${PYTHON_GITLAB_CONFIG}"
        )" || exit 1
    fi
  else
    # shellcheck disable=SC2154
    __value__="$(
      jq -r ".[\"${gitlab}\"].${name} // empty" <<<"${PYTHON_GITLAB_CONFIG}"
      )" || exit 1
  fi

  if [ -n "${__value__}" ] ; then
    echo "${__value__}"
  fi
}

function source_all_files_in_directory {
  local -r folder="$1"
  local    file=

  if [ ! -d "${folder}" ] ; then
    echofatal "Can find directory: '${folder}'"
    exit 1
  fi

  while read -r file ; do
    # shellcheck disable=SC1090
    source "${file}" || exit 1
  done < <( find "${folder}" -type f -name '*.sh' )
}

exit_if_command_is_missing 'jq' 'curl' || exit 1

#
# Bootstrap gitlab-bash-api
#
if is_variable_define_and_not_empty_safe 'PROJECT_HOME' ; then
  if ! is_variable_define_and_not_empty_safe 'GITLAB_BASH_API_PATH' ; then
    declare -gr GITLAB_BASH_API_PATH="${PROJECT_HOME}/bin/gitlab-bash-api"
  fi
  if ! is_variable_define_and_not_empty_safe 'GITLAB_BASH_API_CONFIG_PATH' ; then
    declare -gr GITLAB_BASH_API_CONFIG_PATH="${PROJECT_HOME}/config"
  fi
fi

if ! is_variable_define_and_not_empty_safe 'GITLAB_BASH_API_PATH' ; then
  echofatal "Please set GITLAB_BASH_API_PATH." >&2
  exit 1
fi
if [ ! -d "${GITLAB_BASH_API_PATH}" ]; then
  echofatal "Please set GITLAB_BASH_API_PATH: Directory not found: '${GITLAB_BASH_API_PATH}'" >&2
  exit 1
fi
if ! is_variable_define_and_not_empty_safe 'GITLAB_BASH_API_CONFIG_PATH' ; then
  echofatal "Please set GITLAB_BASH_API_CONFIG_PATH." >&2
  exit 1
fi
if [ ! -d "${GITLAB_BASH_API_CONFIG_PATH}" ]; then
  echofatal "Please set GITLAB_BASH_API_PATH: Directory not found: '${GITLAB_BASH_API_CONFIG_PATH}'" >&2
  exit 1
fi

source_all_files_in_directory "${GITLAB_BASH_API_CONFIG_PATH}" || exit 1

if [ -d "${GITLAB_BASH_API_PATH}/my-config" ]; then
  source_all_files_in_directory "${GITLAB_BASH_API_PATH}/my-config" || exit 1
fi

if is_variable_define_and_not_empty_safe 'GITLAB_BASH_API_CONFIG' ; then
  if [ ! -d "${GITLAB_BASH_API_CONFIG}" ]; then
    echo "GITLAB_BASH_API_CONFIG=${GITLAB_BASH_API_CONFIG} - Folder not found." >&2
    exit 1
  fi

  source_all_files_in_directory "${GITLAB_BASH_API_CONFIG}" || exit 1
fi

#
# Bootstrap from python-gitlab
#
if is_variable_define_and_not_empty_safe 'PYTHON_GITLAB_CFG' && [ -f "${PYTHON_GITLAB_CFG}" ] ; then
  PYTHON_GITLAB_CONFIG_FILE="${PYTHON_GITLAB_CFG}"
elif [ -f ~/.python-gitlab.cfg ] ; then
  PYTHON_GITLAB_CONFIG_FILE=~/.python-gitlab.cfg
else
  PYTHON_GITLAB_CONFIG_FILE='/etc/python-gitlab.cfg'
fi

if [ -f "${PYTHON_GITLAB_CONFIG_FILE}" ] ; then
  declare -gr PYTHON_GITLAB_CONFIG_FILE="${PYTHON_GITLAB_CONFIG_FILE}"
  PYTHON_GITLAB_CONFIG="$( load_configuration )" || exit 1
  declare -gr PYTHON_GITLAB_CONFIG="${PYTHON_GITLAB_CONFIG}"
else
  echofatal 'Config file not found.'
  cat <<EOF >&2
Please create one in one of the following locations: /etc/python-gitlab.cfg, ${HOME}/.python-gitlab.cfg
or set PYTHON_GITLAB_CFG to define a custom file.
EOF
  exit 1
fi

GITLAB_PRIVATE_TOKEN="$( get_python_gitlab_value private_token )" || exit 1
declare -gr GITLAB_PRIVATE_TOKEN="${GITLAB_PRIVATE_TOKEN}"

GITLAB_URL_PREFIX="$( get_python_gitlab_value url | sed -e 's,/$,,g' )" || exit 1
declare -gr GITLAB_URL_PREFIX="${GITLAB_URL_PREFIX}"

if is_variable_define_and_not_empty_safe 'GITLAB_API_VERSION' ; then
  if [ "${GITLAB_API_VERSION}" != 'v4' ] ; then
    if [ "${GITLAB_API_VERSION}" != 'v3' ] ; then
      echofatal 'API V3 is no longer supported.'
    else
      echofatal "API version not supported. ${GITLAB_API_VERSION}"
    fi
    exit 1
  fi
else
  GITLAB_API_VERSION='v4'
fi

#
# Check configuration
#
if ! is_variable_define_and_not_empty_safe 'GITLAB_PRIVATE_TOKEN' ; then
  echo "GITLAB_PRIVATE_TOKEN is missing." >&2
  exit 1
fi

if ! is_variable_define_and_not_empty_safe 'GITLAB_URL_PREFIX' ; then
  echo "GITLAB_URL_PREFIX is missing." >&2
  exit 1
fi

if ! is_variable_define_and_not_empty_safe 'PER_PAGE_MAX' ; then
  # Max value for GitLab is 100
  PER_PAGE_MAX=50
fi

if is_variable_define_and_not_empty_safe 'GITLAB_DEBUG' ; then
  if [ "${GITLAB_DEBUG}" = 'true' ] ; then
    cat <<EOF >&2
--
GITLAB_PRIVATE_TOKEN=${GITLAB_PRIVATE_TOKEN}
GITLAB_URL_PREFIX=${GITLAB_URL_PREFIX}
PER_PAGE_MAX=${PER_PAGE_MAX}
--
EOF
  fi
else
  declare GITLAB_DEBUG=
fi

# shellcheck disable=SC2034
declare -gr LF=$'\n'
# shellcheck disable=SC2034
declare -gr CR=$'\r'
