#!/bin/bash
#
# gitlab-tools:bin/gl-post.sh
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -gr PROJECT_HOME

# shellcheck disable=SC1090
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit 1

function usage {
cat <<EOF
POST Request to GitLab

Usage:
  ${BASH_SOURCE[0]} --help
  ${BASH_SOURCE[0]} [--gitlab GITLAB] --uri GL_URI [--params 'PARAM1=VALUE1&PARAM2=VALUE2']

Action:
  -h,           --help  Display help message and exit
  -u URI,       --uri URI
                        Request URI (/ prefix is optionnal)

Options:
  -p PARAMS,    --params PARAMS
                        Request parameters. Parameters should properly encoded.
  -g GITLAB, --gitlab GITLAB
                        Which configuration section should be used. If not
                        defined, the default selection will be used. (python-gitlab)

Samples:

  Create project: https://docs.gitlab.com/ee/api/projects.html#create-project

    ${BASH_SOURCE[0]} --uri /projects --params 'name=PROJECT_NAME'

  Protected branches

    ${BASH_SOURCE[0]} --uri /projects/PROJECT_ID/repository/branches --params 'name=*-stable&allowed_to_push%5B%5D%5Buser_id%5D=1'
    GITLAB_DEBUG='true' ${BASH_SOURCE[0]} \\
      --uri /projects/418/protected_branches \\
      --params 'name=master&push_access_level=0&merge_access_level=30'

  Protected tags

    ${BASH_SOURCE[0]} --uri /projects/:id/protected_tags
    GITLAB_DEBUG='true' ${BASH_SOURCE[0]} \\
      --uri /projects/418/protected_tags \\
      --params 'name=gitlab-tools-v%2A&create_access_level=40'

EOF
}

function main {
  local api_params=
  local api_url=
  local gitlab=
  local param=

  if [ "$#" -eq 0 ]; then
    usage >&2
    exit 1
  fi

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help') usage; exit 1;;
      '-u'|'--uri') api_url="$( P "$@" )" && shift ;;
      '-p'|'--params') api_params="$( P "$@" )" && shift ;;

      '--gitlab')
        # shellcheck disable=SC2034
        gitlab="$( P "$@" )" && shift
        ;;

      *)
        echofatal "Unknown option: '${param}'" >&2
        usage >&2
        exit 1
        ;;
    esac
  done

  if [ -z "${api_url}" ] ; then
    echofatal "Missing parameter --uri" >&2
    usage >&2
    exit 1
  fi

  # shellcheck disable=SC1090
  source "${PROJECT_HOME}/bin/gitlab-bash-api/init.sh" || exit 1
  # shellcheck disable=SC1090
  source "${GITLAB_BASH_API_PATH}/post.sh" || exit 1

  gitlab_post "${api_url}" "${api_params}"
}

main "$@"
