#!/bin/bash
#
# gitlab-tools:bin/glt-branch.sh
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -r PROJECT_HOME

# shellcheck disable=SC1090
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit 1
# shellcheck disable=SC1090
source "${PROJECT_HOME}/bin/glt-api/branch.sh" || exit 1

function usage {
  cat <<EOF
Manage GitLab branches

Usage:
  ${BASH_SOURCE[0]} action [options] --project-id ID

Action:
  -h, --help            Display help message and exit
  list                  List all branches
  name NAME             Search branch by name
  search STR            Search branches having STR in any field.

Options:
  --project-id ID       Set project ID
  -g GITLAB, --gitlab GITLAB
                        Which configuration section should be used. If not
                        defined, the default selection will be used. (python-gitlab)

Samples:

  ${BASH_SOURCE[0]} list             --project-id PROJECT_ID
  ${BASH_SOURCE[0]} name   'staging' --project-id PROJECT_ID
  ${BASH_SOURCE[0]} search 'staging' --project-id PROJECT_ID

EOF
}

function gitlab_search_branches_by_projectid {
  local -r  gitlab="$1"
  local -ri project_id="$2"
  local -r  search="$3"
  local     first='true'
  local     json=

  echo '['
  while read -r json ; do
    if json="$( grep -i "${search}" <<<"${json}" )"; then
      if [  "${first}" = 'true' ] ; then
        first='false'
      else
        echo ','
      fi
      echo "${json}"
    fi
  done < <( gitlab_get_project_branches_by_projectid "${gitlab}" "${project_id}" | jq -c '.[]' )
  echo ']'
}

function main {
  local action=
  local gitlab=
  local param=
  local project_id=
  local value=

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        usage
        exit 0
        ;;

      'list'        ) action='list' ;;
      'name'        ) action='name' ; value="$( P "$@" )" && shift ;;
      'search'      ) action='search' ; value="$( P "$@" )" && shift ;;

      '--project-id') project_id="$( P "$@" )" && shift ;;
      '--gitlab'    ) gitlab="$( P "$@" )" && shift ;;

      *)
        echofatal "Unknown option: '${param}'"
        usage >&2
        exit 1
        ;;
    esac
  done

  case "${action}" in
    'list')
      if [ -z "${project_id}" ] ; then
        echofatal 'list action imply --project-id'
        usage >&2
        exit 1
      else
        gitlab_get_project_branches_by_projectid "${gitlab}" "${project_id}" | jq .
      fi
      ;;

    'name')
      if [ -z "${value}" ] ; then
        echofatal 'name action imply branch name'
        usage >&2
        exit 1
      elif [ -z "${project_id}" ] ; then
        echofatal 'list action imply --project-id'
        usage >&2
        exit 1
      else
        gitlab_get_project_branch_by_projectid_and_name "${gitlab}" "${project_id}" "${value}" | jq .
      fi
      ;;

    'search')
      if [ -z "${value}" ] ; then
        echofatal 'search action imply search string'
        usage >&2
        exit 1
      elif [ -z "${project_id}" ] ; then
        echofatal 'search action imply --project-id'
        usage >&2
        exit 1
      else
        gitlab_search_branches_by_projectid "${gitlab}" "${project_id}" "${value}" | jq .
      fi
      ;;

    *)
      echofatal 'Missing action.'
      usage >&2
      exit 1
      ;;
  esac
}

main "$@" || exit 1
