#!/bin/bash -e
#
# my-tools:bin/glx-find-wrong-remote-urls.sh
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -r PROJECT_HOME

# shellcheck disable=SC1090
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit 1
# shellcheck disable=SC1090
source "${PROJECT_HOME}/bin/git-api/git-api.sh" || exit 1
# shellcheck disable=SC1090
source "${PROJECT_HOME}/bin/glt-api/project.sh" || exit 1

function usage {
  cat <<EOF
Looking for local git repositories with wrong remotes URLs

Usage:
  ${BASH_SOURCE[0]} --help
  ${BASH_SOURCE[0]} [--gitlab GITLAB] [--sources-directory DIR]

Options:
  -d DIR,       --dir DIR,      --sources-directory DIR
                    Use given directory, instead of current directory.
  -g GITLAB,    --gitlab GITLAB
                    Which configuration section should be used. If not
                    defined, the default selection will be used. (python-gitlab)
  -v,           --verbose
                    Display what's append.

EOF
}

function echoinfo_verbose {
  if [ "${param_verbose}" = 'true' ] ; then
    echoinfo "$@" >&2
  fi
}

function echowarn_verbose {
  if [ "${param_verbose}" = 'true' ] ; then
    echowarn "$@" >&2
  fi
}

function init_project_list_summary {
  local cache=
  echoinfo_verbose 'Loading GitLab project list...'

  cache="$(
    gitlab_get_all_projects "${gitlab}" |
      jq -c '[ .[] |
        { id: .id, name: .name, path: .path, ssh_url_to_repo: .ssh_url_to_repo, http_url_to_repo: .http_url_to_repo }
        ]'
    )"

  if [ -z "${cache}" ] ; then
    echofatal 'Error while getting GitLab project list'
    exit 1
  fi

  echo "${cache}"
}

function get_project_list_summary {
  if ! is_variable_define_and_not_empty_safe __project_list_summary_cache__ ; then
    echofatal 'Error while getting GitLab project list: cache not initialised.'
    exit 1
  fi

  echo "${__project_list_summary_cache__}"
}

function check_remote_url {
  local -r git_directory="$1"
  local -r remote_name="$2"
  local -r remote_url="$3"
  local    project_json=
  local    path=
  local    guest_project_name=
  local    ssh_url_to_repo=
  local    http_url_to_repo=
  local -a found_projects_json=()
  local    project_macth_found='false'
  local    url_ok='false'

  guest_project_name="$( basename "${remote_url%.git}" )"

  echoinfo_verbose "Handle: '${remote_name}' : '${remote_url%.git}' - name?: '${guest_project_name}'"

  while read -r project_json ; do
    path="$( jq -r '.path' <<<"${project_json}" )" || exit 1

    if [ "${path}" = "${guest_project_name}" ] ; then
      # Project name match
      project_macth_found='true'
      found_projects_json+=( "${project_json}" )
      ssh_url_to_repo="$( jq -r '.ssh_url_to_repo' <<<"${project_json}" )" || exit 1

      if [ "${ssh_url_to_repo}" = "${remote_url}" ] ; then
        echoinfo_verbose "SSH URL Ok"
        url_ok='true'
      else
        http_url_to_repo="$( jq -r '.http_url_to_repo' <<<"${project_json}" )" || exit 1

        if [ "${http_url_to_repo}" = "${remote_url}" ] ; then
          echoinfo_verbose "HTTP URL Ok"
          url_ok='true'
        fi
      fi
    fi
  done < <( get_project_list_summary | jq -c '.[]' )

  if [ "${url_ok}" = 'true' ] ; then
    return 0 # OK
  else
    echowarn_verbose "'${remote_url}' not found while looking for project path: '${guest_project_name}'"
  fi

  # ERROR
  cat <<EOF
{
  "directory": "${git_directory}",
  "url-config-okay": ${url_ok},
  "candidate-found": ${project_macth_found},
  "remote": {
    "name": "${remote_name}",
    "url": "${remote_url}"
  },
  "urls": [
EOF
  if [ "${project_macth_found}" = 'false' ] ; then
    echoerror "No candidate for: '${git_directory}' - '${remote_name}' - '${remote_url}'"
  else
    echowarn_verbose "Candidate for: '${git_directory}' - '${remote_name}' - '${remote_url}'"
    local first='true'

    for project_json in "${found_projects_json[@]}" ; do
      [ "${first}" = 'true' ] && first='false' || echo ','

      ssh_url_to_repo="$( jq -r '.ssh_url_to_repo' <<<"${project_json}" )" || exit 1
      http_url_to_repo="$( jq -r '.http_url_to_repo' <<<"${project_json}" )" || exit 1
    cat <<EOF
    {
      "ssh": "${ssh_url_to_repo}",
      "http": "${http_url_to_repo}"
    }
EOF
    done
  fi

  cat <<EOF
  ]
}
EOF
}

function check_all_remote_urls {
  local -r git_directory="$1"

  while read -r remote_name ; do
    remote_url="$( get_remote_url "${git_directory}" "${remote_name}" )" || exit 1

    result="$( check_remote_url "${git_directory}" "${remote_name}" "${remote_url}" )" || exit 1
    if [ -n "${result}" ] ; then
      [ "${first}" = 'true' ] && first='false' || echo ','
      echo "${result}"
    fi

  done < <( get_all_remote "${git_directory}" )
}

function check_all_git_directories {
  local -r sources_home="$1"
  local    git_directory=
  local    remote_name=
  local    remote_url=
  local    first='true'
  local    result=

  cat <<EOF
[
EOF
  while read -r git_directory ; do
    echoinfo_verbose "Checking: ${git_directory}"

    check_all_remote_urls "${git_directory}"
  done < <( find_git_directories "${sources_home}" )
  cat <<EOF
]
EOF
}

function main {
  local gitlab=
  local param=
  local param_cache=
  local param_sources_home=
  local param_verbose='false'

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        usage
        exit 0
        ;;

      '--gitlab')
        gitlab="$( P "$@" )" && shift
        ;;

      '-v'|'--verbose')
        param_verbose='true'
        ;;

      '-d'|'--dir'|'--sources-directory')
        param_sources_home="$( P "$@" )" && shift
        ;;

      '-c'|'--cache')
        param_cache="$( P "$@" )" && shift
        ;;

      *)
        echofatal "Unknown option: '${param}'"
        usage >&2
        exit 1
        ;;
    esac
  done

  if [ -z "${param_sources_home}" ] ; then
    param_sources_home="$( pwd )" || exit 1
  fi

  # Init cache.
  __project_list_summary_cache__="$( init_project_list_summary )" || exit 1
  declare -grx __project_list_summary_cache__
  if [ -n "${param_cache}" ] ; then
    jq '.' <<<"${__project_list_summary_cache__}" >"${param_cache}" || exit 1
  fi

  local bad_json=
  bad_json="$( mktemp )" || exit 1

  if ! check_all_git_directories "${param_sources_home}" | tee "${bad_json}" | jq '.' ; then
    cat "${bad_json}"
  fi
  rm "${bad_json}"
}

main "$@"
