#!/bin/bash
#
# gitlab-tools:bin/glx-projects-configuration.sh
#
# https://docs.gitlab.com/ee/api/projects.html#edit-project
#
# Configuration - BEGIN
PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -r PROJECT_HOME

# shellcheck disable=SC1090
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit 1

if is_variable_define_and_not_empty_safe 'GITLAB_BASH_API_PATH' ; then
  # shellcheck disable=SC1090
  source "${GITLAB_BASH_API_PATH}/init.sh" || exit 1
elif [ ! -f "${PROJECT_HOME}/bin/gitlab-bash-api/init.sh" ]; then
  echofatal "gitlab-bash-api/init.sh not found! - Please set GITLAB_BASH_API_PATH" >&2
  exit 1
else
  # shellcheck disable=SC1090
  source "${PROJECT_HOME}/bin/gitlab-bash-api/init.sh" || exit 1
fi
# Configuration - END

function usage {
cat <<EOF
xxxxx

Usage:
  ${BASH_SOURCE[0]} --help
  ${BASH_SOURCE[0]} --audit [--config-dir DIR]
  ${BASH_SOURCE[0]} --audit [--config-dir DIR] --group-id GID
  ${BASH_SOURCE[0]} --audit [--config-dir DIR] --project-id GID
  ${BASH_SOURCE[0]} --apply [--config-dir DIR]
  ${BASH_SOURCE[0]} --apply [--config-dir DIR] --group-id GID
  ${BASH_SOURCE[0]} --apply [--config-dir DIR] --project-id GID

Action:
  -h,           --help  Display help message and exit
  -A,           --audit Audit current configuration
  -C,           --apply Apply new configuration

  -D DIR,       --config-dir DIR
                        Set project configuration directory

  -G GID,        --group-id GID
                        Audit or apply on all projects of given group ID.

  -P PID,       --project-id PID
                        Audit or apply for given project ID only

EOF
}

declare -g __projects_config_audit_filter__=

function __projects_config_audit_build_filter__ {
  local -ra keys_to_remove=(
_links
archived
avatar_url
ci_config_path
ci_default_git_depth
created_at
creator_id
description
empty_repo
forks_count
http_url_to_repo
id
import_status
last_activity_at
name
name_with_namespace
namespace
open_issues_count
path
path_with_namespace
permissions
readme_url
readme_url
readme_url
ssh_url_to_repo
star_count
web_url
)
  __projects_config_audit_filter__='.[]'

  for key in "${keys_to_remove[@]}"; do
    __projects_config_audit_filter__+=" | del( .${key} )"
  done
  #  __projects_config_audit_filter__+=''
}

function projects_config_audit {
  local -r project_id="$1"
  local -r project_config_json_file="$2"
  local -r audit_dir="${project_config_json_file}.d"
  local -r audit_file="${audit_dir}/${project_id}.json"

  [ ! -d "${audit_dir}" ] && mkdir "${audit_dir}"

  "${PROJECT_HOME}/bin/glt-project.sh" id "${project_id}" |
    jq "${__projects_config_audit_filter__}" >"${audit_file}"
}

function projects_config_apply {
  local -r project_id="$1"
  local -r project_config_json_file="$2"

  echoinfo "Apply: ${project_config_json_file} on ${project_id}: ${project_url}"

  "${PROJECT_HOME}/bin/glt-project.sh" modify-json "${project_id}" "${project_config_json_file}"
}

function projects_config_for_group {
  local -r mode="$1"
  local -r group_id="$2"
  local -r project_config_json_file="$3"
  local    projects_json=
  local    project_json=
  local    project_url=
  local    project_id=

  projects_json="$(
    "${PROJECT_HOME}/bin/glt-project.sh" group "${group_id}" | jq '[ .[] | { id: .id, web_url: .web_url } ] | sort_by( .id )'
    )"

  while read -r project_json ; do
    project_url="$( jq -r '.web_url' <<<"${project_json}" )"
    project_id="$( jq -r '.id' <<<"${project_json}" )"

    echoinfo "P:${project_id}:${project_url}"

    case "${mode}" in
      'apply')
        projects_config_apply "${project_id}" "${project_config_json_file}"
        ;;

      *)
        projects_config_audit "${project_id}" "${project_config_json_file}"
        ;;
    esac
  done < <( jq -c '.[]' <<<"${projects_json}" )
}

function projects_config {
  local -r  mode="$1"
  local -ri project_id="$2"
  local -r  project_config_json_file="$3"

  case "${mode}" in
    'apply')
      projects_config_apply "${project_id}" "${project_config_json_file}"
      ;;

    *)
      projects_config_audit "${project_id}" "${project_config_json_file}"
      ;;
  esac
}

function get_project_config_json_file_for_group {
  local -r  project_configuration_directory="$1"
  local -ri group_id="$2"

  echo "${project_configuration_directory}/project_config.group.${group_id}.json"
}

function get_project_config_json_file_for_project {
  local -r  project_configuration_directory="$1"
  local -ri project_id="$2"

  echo "${project_configuration_directory}/project_config.${project_id}.json"
}

function projects_config_by_group {
  local -r mode="$1"
  local -r project_configuration_directory="$2"
  local    groups_json=
  local    group_json=
  local    group_url=
  local -i group_id=0
  local    project_config_json_file=
  local    skip=

  groups_json="$(
    "${PROJECT_HOME}/bin/glt-group.sh" list | jq '[ .[] | { id: .id, web_url: .web_url } ] | sort_by( .id )'
    )"

  while read -r group_json ; do
    group_url="$( jq -r '.web_url' <<<"${group_json}" )"
    group_id="$( jq -r '.id' <<<"${group_json}" )"

    case "${mode}" in
      'apply')
        project_config_json_file="$( get_project_config_json_file_for_group "${project_configuration_directory}" "${group_id}" )"

        if [ -f "${project_config_json_file}" ] ; then
          echoinfo "Apply configuration from: '${project_config_json_file}'"
          skip='false'
        else
          echowarn "No configuration file for group:${group_id} - File not found: '${project_config_json_file}'"
          skip='true'
        fi
        ;;

      *)
        project_config_json_file="$(
          get_project_config_json_file_for_group "${project_configuration_directory}" "${group_id}"
          )" || return 1
        echoinfo "Audit configuration to: '${project_config_json_file}'"
        skip='false'
        ;;
    esac

    echoinfo "G:${group_id}:${group_url}"

    if [ "${skip}" = 'false' ] ; then
      projects_config_for_group "${mode}" "${group_id}" "${project_config_json_file}"
    fi
  done < <( jq -c '.[]' <<<"${groups_json}" )
}

function main {
  local param=
  local param_groupid=
  local param_mode=
  local param_project_configuration_directory=
  local param_projectid=

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help') usage; exit 1;;
      '-A'|'--audit') param_mode='audit' ;;
      '-C'|'--apply') param_mode='apply' ;;
      '-D'|'--config-dir') param_project_configuration_directory="$( P "$@" )" && shift ;;
      '-G'|'--group-id') param_groupid="$( P "$@" )" && shift ;;
      '-P'|'--project-id') param_projectid="$( P "$@" )" && shift ;;

    *)
      echofatal "Unknown option: '${param}'" >&2
      usage >&2
      exit 1
      ;;
    esac
  done

  case "${param_mode}" in
    'audit') __projects_config_audit_build_filter__ ;;
    'apply') ;;

    '')
      echofatal '--audit or --apply is required'
      usage >&2
      exit 1
      ;;

    *)
      echoerror "param_mode='${param_mode}' - Wrong value"
      exit 1
      ;;
  esac

  if [ -z "${param_project_configuration_directory}" ] ; then
    param_project_configuration_directory='.'
  elif [ ! -d "${param_project_configuration_directory}" ] ; then
    echofatal "Directory not found: '${param_project_configuration_directory}'"
    exit 1
  fi

  if [ -n "${param_groupid}" ] && [ -n "${param_projectid}" ] ; then
    echofatal "--group-id and --project-id can't be used at the same time"
    usage >&2
    exit 1
  fi

  declare -gr MESSAGE_API_HIDE_DATE='true'
  export MESSAGE_API_HIDE_DATE

  declare -gr CACHE_TIMEOUT='-300 seconds' # 5 minutes
  export CACHE_TIMEOUT

  if [ -n "${param_groupid}" ] ; then
    if is_positive_integer "${param_groupid}" ; then
      local project_config_json_file=
      project_config_json_file="$(
        get_project_config_json_file_for_group "${param_project_configuration_directory}" "${param_groupid}"
        )" || return 1

      projects_config_for_group "${param_mode}" "${param_groupid}" "${project_config_json_file}"
    else
      echofatal "--group-id must be an integer. Found '${param_groupid}'"
      exit 1
    fi
  elif [ -n "${param_projectid}" ] ; then
    if is_positive_integer "${param_projectid}" ; then
      local project_config_json_file=
      project_config_json_file="$(
        get_project_config_json_file_for_project "${param_project_configuration_directory}" "${param_projectid}"
        )" || return 1

      projects_config "${param_mode}" "${param_projectid}" "${project_config_json_file}"
    else
      echofatal "--project-id must be an integer. Found '${param_projectid}'"
      exit 1
    fi
  else
    projects_config_by_group "${param_mode}" "${param_project_configuration_directory}"
  fi
}

main "$@"
