#!/bin/bash
#
# gitlab-tools:bin/glx-projects-protected-master-branch.sh
#

# Configuration - BEGIN
PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -r PROJECT_HOME

# shellcheck disable=SC1090
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit 1

if is_variable_define_and_not_empty_safe 'GITLAB_BASH_API_PATH' ; then
  # shellcheck disable=SC1090
  source "${GITLAB_BASH_API_PATH}/init.sh" || exit 1
elif [ ! -f "${PROJECT_HOME}/bin/gitlab-bash-api/init.sh" ]; then
  echofatal "gitlab-bash-api/init.sh not found! - Please set GITLAB_BASH_API_PATH" >&2
  exit 1
else
  # shellcheck disable=SC1090
  source "${PROJECT_HOME}/bin/gitlab-bash-api/init.sh" || exit 1
fi
# Configuration - END

# Script start here
declare -gr  PROTECTED_DEFAULT='false'
declare -gri MAX_DEFAULT='100'

function usage {
cat <<EOF
Get Request to GitLab

Usage:
  ${BASH_SOURCE[0]} --help
  ${BASH_SOURCE[0]} [protected true|false]

Action:
  -h,           --help  Display help message and exit
  -v BOOL,      --value BOOL, protected BOOL
                        Return project/protected where protected have given value
                        (default is: ${PROTECTED_DEFAULT})
  -g GROUPID,   --group-id GROUPID
                        Limit research to GROUPID
  -M MAX,       --max-results
                        Limit results to MAX entries
                        (default is: ${MAX_DEFAULT})

Samples:

    ${BASH_SOURCE[0]}
    ${BASH_SOURCE[0]} protected true

EOF
}

function is_integer {
  local -r interger="$1"

  if [[ "${interger}" =~ ^[0-9]+$ ]] ; then
    return 0
  else
    return 1
  fi
}

function do_get_projects_without_protected_master_branch {
  local -ri group_id="$1"
  local -r  master_branch_name="$2"
  local -r  expected_branch_protected="$3"
  local -ri max_results="$4"
  local -i  count_result=1
  local     first='true'
  local     project_json=
  local     project_id=
  local     project_default_branch=
  local     branch_protected=

  echo '['
  while read -r project_json ; do
    project_id="$( jq '.id' <<<"${project_json}" )" || exit 1
    project_default_branch="$( jq -r '.default_branch' <<<"${project_json}" )" || exit 1

    branch_json="$(
      "${PROJECT_HOME}/bin/gl-get.sh" -u "projects/${project_id}/repository/branches/${project_default_branch}" |
      jq -c 'del( .commit )'
      )" || exit 1
    branch_protected="$( jq '.protected' <<<"${branch_json}" )" || exit 1

    if [ "${branch_protected}" = "${expected_branch_protected}" ] ; then
      project_web_url="$( jq -r '.web_url' <<<"${project_json}" )" || exit 1

      if [ "${first}" = 'true' ] ; then
        first='false'
      else
        echo ','
      fi
      cat <<JSON
  {
    "project_id": "${project_id}",
    "web_url": "${project_web_url}",
    "default_branch": "${project_default_branch}",
    "branch": ${branch_json}
  }
JSON
      count_result=$((count_result+1))
      if [ "${count_result}" -gt "${max_results}" ] ; then
        echowarn "Max results: ${max_results}" >&2
        break
      fi
    fi

  done < <( get_projects_list "${group_id}" )
  echo ']'
}

function get_projects_list {
  local -ri group_id="$1"

  if [ "${group_id}" -eq 0 ] ; then
    "${PROJECT_HOME}/bin/glt-project.sh" list | jq -c '.[]'
  else
    "${PROJECT_HOME}/bin/glt-project.sh" group "${group_id}" | jq -c '.[]'
  fi
}

function main {
  local expected_branch_protected="${PROTECTED_DEFAULT}"
  local group_id=0
  local master_branch_name= # TODO: allow to fix name of branch
  local max_results="${MAX_DEFAULT}"
  local param=

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
    '-h'|'--help') usage; exit 1;;
    '-v'|'--value'|'protected')
      expected_branch_protected="$( P "$@" )" && shift
      ;;

    '-M'|'--max-results')
      max_results="$( P "$@" )" && shift
      ;;

    '-g'|'--group-id')
      group_id="$( P "$@" )" && shift
      ;;

    *)
      echofatal "Unknown option: '${param}'" >&2
      usage >&2
      exit 1
      ;;
    esac
  done

  if ! is_boolean "${expected_branch_protected}" ; then
    echofatal "Value must be a boolean: '${expected_branch_protected}'" >&2
    usage >&2
    exit 1
  fi

  if ! is_integer "${max_results}" ; then
    echofatal "--max-results value must be an integer: '${max_results}'" >&2
    usage >&2
    exit 1
  fi

  if ! is_integer "${group_id}" ; then
    echofatal "--group-id value must be an integer: '${group_id}'" >&2
    usage >&2
    exit 1
  fi

  do_get_projects_without_protected_master_branch \
    "${group_id}" \
    "${master_branch_name}" \
    "${expected_branch_protected}" \
    "${max_results}" |
    jq . || exit 1
}

main "$@"
